/*
     Object Oriented Programming using C++,
     Solution Manual and Applied Examples.
	    written by Dr. Farschad Torabi
     K. N. Toosi University of Technology  
	           September, 2013

                Chapter 25
      Code written by: Ali Sohani
*/

#include<iostream>
using namespace std;

//Define "Manba" Class For Using In "ElectricElement":
class Manba
{
public :
  double Voltage;
  double Power;
};

//Define "ElectricElement" Class As Base Parent Class:
//Notice This Class Is Abstract:
class ElectricElement
{
public :
  //Class Attributes:
  double Ra;
  double Rf;
  double Il;
  double If;
  double Ia;
  double Ea;
  double Vb;
  double Culoss;
  double Prot;
  double w;
  double Wm;
  double Te;
  double efficiency;
  Manba Vt;
  //Define Constructor:
  ElectricElement()
  {
    Ra=0;
    Rf=0;
    Il=0;
    If=0;
    Ia=0;
    Ea=0;
    Vb = 0;
    GetData();
  }
  //Define Non-Abstract Methods:
  void Tabdil(double Wm)
  {
    w = 2*3.1415*Wm/60;
  }
  void PrintData()
  {
    cout<<"---        Results        ---"<<endl;
    cout<<"Il = "<<Il<<" (A)"<<endl;
    cout<<"If = "<<If<<" (A)"<<endl;
    cout<<"Ia = "<<Ia<<" (A)"<<endl;
    cout<<"Ea = "<<Ea<<" (V)"<<endl;
    cout<<"Te = "<<Te<<" (N-m)"<<endl;
    cout<<"Efficiency = "<<efficiency<<" %"<<endl;
  }
  void GetData()
  {
    cout<<"Please Enter Vt (Volt): "; 
    cin>>Vt.Voltage;     
    cout<<"Please Enter Power Of Voltage Source (Watt): ";
    cin>>Vt.Power;    
    cout<<"Please Enter Ra (Ohm): ";
    cin>>Ra;    
    cout<<"Please Enter Rf (Ohm): ";
    cin>>Rf;    
    cout<<"Please Enter Vb (Volt): ";
    cin>>Vb;    
    cout<<"Please Enter Prot (Watt): ";
    cin>>Prot;  
    cout<<"Please Enter Wm (rpm): ";
    cin>>Wm;    Tabdil(Wm); 
  }
  void Calculate()
  {
    CalIl();
    CalIf();
    CalIa();
    CalCuloss();
    CalEa();
    CalTe();
    CalEfficiency();
  }
  void CalTe()
  {
    Te = Ea*Ia/w;
  }
  virtual void CalEfficiency()
  {
    efficiency = (Vt.Power)*100/(Ea*Ia+Prot);
  }
  //Define Abstract Methods:
  virtual void CalIl() = 0;
  virtual void CalIf() = 0;
  virtual void CalIa() = 0;
  virtual void CalCuloss() = 0;
  virtual void CalEa() = 0;
};

//Define "SeparateExcitedGenerator" Class As The Child Of "ElectricElement" Class:
class SeparateExcitedGenerator : public ElectricElement
{
public :
  //Attributes:
  double K1;
  double ITahrik;
  //Methods:
  virtual void CalIl()
  {
    Il = Vt.Power/Vt.Voltage;
  }
  virtual void CalIf()
  {
  }
  void CalITahrik()
  {
    cout<<"Please Enter Machine Constant: ";    cin>>K1;
    ITahrik = Ea/(K1*w);
    cout<<"I Tahrik = "<<ITahrik<<" (A)"<<endl;
  }
  virtual void CalIa()
  {
    Ia = Il;
  }
  virtual void CalEa()
  {
    Ea = Vt.Voltage + Ra*Ia +Vb;
  }
  virtual void CalCuloss()
  {
    Culoss = Ea*Ia - Vt.Voltage*Il;
  }
};

//Define "ShuntGenerator" Class As The Child Of "ElectricElement" Class:
class ShuntGenerator : public ElectricElement
{
public :
  //Methods:
  virtual void CalIl()
  {
    Il = Vt.Power/Vt.Voltage;
  }
  virtual void CalIf()
  {
    If = Vt.Voltage/Rf;
  }
  virtual void CalIa()
  {
    Ia = Il + If ;
  }
  virtual void CalEa()
  {
    Ea = Vt.Voltage + Ra*Ia + Vb;
  }
  virtual void CalCuloss()
  {
    Culoss = Ea*Ia - Vt.Power;
  }
};

//Define "SeriGenerator" Class As The Child Of "ElectricElement" Class:
class SeriGenerator : public ElectricElement
{
public :
  //Methods:
  virtual void CalIl()
  {
    Il = Vt.Power/Vt.Voltage;
  }
  virtual void CalIf()
  {
    If = Il;
  }
  virtual void CalIa()
  {
    Ia = Il;
  }
  virtual void CalEa()
  {
    Ea = Vt.Voltage + (Ra+Rf)*Ia +Vb ;
  }
  virtual void CalCuloss()
  {
    Culoss = Ea*Ia -(Ra+Rf)*Ia*Ia;
  }
};

//Define "CompoundGenerator" Class As The Child Of "ElectricElement" Class:
//Notice That This Class Is Abstract.
class CompoundGenerator : public ElectricElement
{
public :
  double Rs;
  CompoundGenerator()
  {
    cout<<"Please Enter Rs (Ohm): ";
    cin>>Rs;  
  }
  virtual void CalIl() = 0;
  virtual void CalIf() = 0;
  virtual void CalIa() = 0;
  virtual void CalCuloss() = 0;
  virtual void CalEa() = 0;
};

//Define "LongShuntCompoundGenerator" Class As The Child Of "CompoundGenerator" Class:
class LongShuntCompoundGenerator : public CompoundGenerator
{
  //Methods:
  virtual void CalIl()
  {
    Il = Vt.Power/Vt.Voltage;
  }
  virtual void CalIf()
  {
    If = Vt.Voltage/Rf;
  }
  virtual void CalIa()
  {
    Ia = Il + If;
  }
  virtual void CalEa()
  {
    Ea = Vt.Voltage + (Ra+Rs)*Ia +Vb ;
  }
  virtual void CalCuloss()
  {
    Culoss = Ea*Ia - Vt.Power;
  }
};

//Define "ShortShuntCompoundGenerator" Class As The Child Of "CompoundGenerator" Class:
class ShortShuntCompoundGenerator : public CompoundGenerator
{
  //Methods:
  virtual void CalIl()
  {
    Il = Vt.Power/Vt.Voltage;
  }
  virtual void CalIf()
  {
    If = Vt.Voltage/Rf;
  }
  virtual void CalIa()
  {
    Ia = Il + If;
  }
  virtual void CalEa()
  {
    Ea = Vt.Voltage + Rs*Il + (Ra+Rs)*Ia +Vb ;
  }
  virtual void CalCuloss()
  {
    Culoss = Ea*Ia - Vt.Power;
  }
};

//Define "ShuntMotor" Class As The Child Of "ElectricElement" Class:
class ShuntMotor : public ElectricElement
{
public :
  //Methods:
  virtual void CalIl()
  {
    Il = Vt.Power/Vt.Voltage;
  }
  virtual void CalIf()
  {
    If = Vt.Voltage/Rf;
  }
  virtual void CalIa()
  {
    Ia = Il - If ;
  }
  virtual void CalEa()
  {
    Ea = Vt.Voltage - Ra*Ia - Vb;
  }
  virtual void CalCuloss()
  {
    Culoss = Ea*Ia - Vt.Power;
  }
  void CalMEfficiency()
  {
    efficiency = ((Ea*Ia-Prot)*100/(Vt.Power));
  }
};

int main()
{
  ElectricElement * PEE;
  int k = 5;

  //While Is There Because We Can Repeat Program.
  while(k==5)
  {
    cout<<"--- Welcome To DCMachineMohasebat Program  ---"<<endl;
    cout<<"--- If You Have DCGenerator Please Enter 1 ---"<<endl;
    cout<<"--- If You Have   DCMotor   Please Enter 2 ---"<<endl;

    int n;
    cin>>n;
    switch(n)
    {
    case(1) :
      {
        cout<<"---          Choose Your DCGenerator Type          ---"<<endl;
        cout<<"--- SeparateExcited 1, Shunt 2, Seri 3, Compound 4 ---"<<endl;
        int i;
        cin>>i;
        if(i==1)
        {
          PEE = new SeparateExcitedGenerator ;
          PEE->Calculate();
          ((SeparateExcitedGenerator*)PEE)->CalEfficiency();
          ((SeparateExcitedGenerator*)PEE)->CalITahrik();
          PEE->PrintData();
        }
        if(i==2)
        {
          PEE = new ShuntGenerator ;
          PEE->Calculate();
          ((ShuntGenerator*)PEE)->CalEfficiency();
          PEE->PrintData();
        }
        if(i==3)
        {
          PEE = new SeriGenerator ;
          PEE->Calculate();
          ((SeriGenerator*)PEE)->CalEfficiency();
          PEE->PrintData();
        }
        if(i==4)
        {
          cout<<"--- Short Compound = 1, Long Compound = 2 ---"<<endl;
          int t;
          cin>>t;
          if(t==1)
          {
            PEE = new ShortShuntCompoundGenerator ;
            PEE->Calculate();
            ((ShortShuntCompoundGenerator*)PEE)->CalEfficiency();
            PEE->PrintData();
          }
          if(t==2)
          {
            PEE = new LongShuntCompoundGenerator;
            PEE->Calculate();
            ((LongShuntCompoundGenerator*)PEE)->CalEfficiency();
            PEE->PrintData();
          }
        }
        break;	
      }
    case(2) :
      {
        PEE = new ShuntMotor ;
        PEE->Calculate();
        ((ShuntMotor*)PEE)->CalMEfficiency();
        PEE->PrintData();
        break;
      }
    default :
      {
        cout<<"--- You Entered an Invalid Number, Please Try Again ---"<<endl;
      }
    }

    cout<<"--- Continue = 5, Exit Program = Any Other Number ---"<<endl;
    cin>>k;
    system("cls");
  }
  system("pause");
  return 0;
}